@@ -1,8 +1,9 @@
 import logging
-import traceback
+import warnings
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, List, Optional, Sequence, Set, TypeVar
+from typing import TYPE_CHECKING, Optional, Sequence, Set, TypeVar
 
+from .common.logging import log_exception
 from .common.registrable import Registrable
 from .common.util import import_extra_module
 from .step_graph import StepGraph
@@ -38,6 +39,11 @@ class Executor(Registrable):
     An ``Executor`` is a class that is responsible for running steps and caching their results.
 
     This is the base class and default implementation, registered as "default".
+
+    .. note::
+        The ``parallelism`` parameter has no effect with this default :class:`Executor`,
+        but is part of the API because most subclass implementations allow configuring
+        parallelism.
     """
 
     default_implementation = "default"
@@ -46,9 +52,11 @@ class Executor(Registrable):
         self,
         workspace: Workspace,
         include_package: Optional[Sequence[str]] = None,
+        parallelism: Optional[int] = None,
     ) -> None:
         self.workspace = workspace
         self.include_package = include_package
+        self.parallelism = parallelism
 
     def execute_step(self, step: "Step") -> None:
         # Import included packages to find registered components.
@@ -65,16 +73,21 @@ class Executor(Registrable):
         self, step_graph: StepGraph, run_name: Optional[str] = None
     ) -> ExecutorOutput:
         """
-        Execute a :class:`tango.step_graph.StepGraph`. This attempts to execute
+        Execute a :class:`~tango.step_graph.StepGraph`. This attempts to execute
         every step in order. If a step fails, its dependent steps are not run,
         but unrelated steps are still executed. Step failures will be logged, but
         no exceptions will be raised.
         """
+        if self.parallelism is not None:
+            warnings.warn(
+                "The 'parallelism' parameter has no effect with the default Executor. "
+                "If you want to run steps in parallel, consider using the MulticoreExecutor.",
+                UserWarning,
+            )
 
         successful: Set[str] = set()
         failed: Set[str] = set()
         not_run: Set[str] = set()
-        error_tracebacks: List[str] = []
         uncacheable_leaf_steps = step_graph.uncacheable_leaf_steps()
 
         for step in step_graph.values():
@@ -88,14 +101,26 @@ class Executor(Registrable):
                 try:
                     self.execute_step(step)
                     successful.add(step.name)
-                except Exception:
+                except Exception as exc:
                     failed.add(step.name)
-                    error_tracebacks.append(traceback.format_exc())
-
-        for stacktrace in error_tracebacks:
-            logger.error(stacktrace)
+                    log_exception(exc, logger)
 
         return ExecutorOutput(successful=successful, failed=failed, not_run=not_run)
 
+    # NOTE: The reason for having this method instead of just using `execute_step()` to run
+    # a single step is that the certain executors, such as the BeakerExecutor, need to
+    # serialize steps somehow, and the easiest way to serialize a step is by serializing the
+    # whole step config (which can be accessed via the step graph).
+
+    def execute_sub_graph_for_step(
+        self, step_graph: StepGraph, step_name: str, run_name: Optional[str] = None
+    ) -> None:
+        """
+        Execute the sub-graph associated with a particular step in a
+        :class:`~tango.step_graph.StepGraph`.
+        """
+        step = step_graph[step_name]
+        self.execute_step(step)
+
 
 Executor.register("default")(Executor)
